---
outline: [2, 4]
---
# Server <Badge type="tip" text="^0.2.0" />

Package used on a server-side of your Rage:MP Server

## Usage

```ts
import { fw } from '@entityseven/rage-fw-server'
```

Only usable in server environment. For usage in [Client](./client) and [Browser](./browser) refer to their sections

## Declaration

```ts
fw = {
  event: Server,
  player: Player,
  system: {
    log: Logger
  },
  rpc: Rpc
}
```

Further documentation will describe the fields included in `fw`

## Server

Server-side interactions

### register

Registers a server event with an associated callback

```ts
fw.event.register("playerJoin", (player) => {
    // do something
})

// Registering an event with middlewares
fw.event.register("playerJoin", (player) => {
    // do something
}, {
    middlewares: [...] // <- your middlewares here
})

// or

fw.event.register("playerJoin", (player) => {
    // do something
}, {
    middlewares: {
        executables: [...], // <- your middlewares here
        onError: (msg) => void msg // <- error handling here
    }
})
```

### unregister

Unregisters a server event, removing the associated callback

```ts
fw.event.unregister('playerJoin')
```

### trigger

Triggers registered server event. Formerly known as `call` or `emit`

```ts
fw.event.trigger('serverEventName', ['arg1', 2])
```

## Player

Handles event manipulations that require player to be present in context

### triggerClient

Triggers a client event from the server with arguments from shared types. Formerly known as `callClient` or `emitClient`

```ts
// without args
fw.player.triggerClient('clientEventName')
// with args
fw.player.triggerClient('clientEventName', ['message to client'])
```

### triggerBrowser

Triggers a browser event from the server with arguments from shared types. Formerly known as `callBrowser` or `emitBrowser`

```ts
// without args
fw.player.triggerBrowser('browserEventName')
// with args
fw.player.triggerBrowser('browserEventName', ['message to browser'])
```

## System

Handles functions used to interact with system environment

### Logger

Used to log to a client in-game console

#### info

Informational logs. Colored in white

```ts
fw.system.log.info('some information to be logged')
```

#### warn

Warning logs. Colored in yellow

```ts
fw.system.log.warn('warning message')
```

#### error

Error logs. Colored in red

```ts
fw.system.log.info('some error information')
```

### Rpc

`rage-fw-rpc` instance used under the hood. It is highly recommended to use this one if you need it instead of creating a new instance
