---
outline: [2, 4]
---
# Features <Badge type="tip" text="^0.3.0" />

## Event Middlewares

Rage FW offers you to add middlewares when registering events in Client and Server environments, to check if callback should be executed or should not. Core class functionality is not exposed to user, but some types are for easier code managing. Here are some key points (in context of Server environment, but also applicable to Client)

### Declaration

```ts
register(
    eventName,
    callback,
    options?: {
        middlewares?: RageFW_MiddlewareOptions
    },
)

type RageFW_MiddlewareOptions =
    | RageFW_MiddlewareFunction[]
    | {
          executables: RageFW_MiddlewareFunction[]
          onError: (error: string) => unknown
      }

type RageFW_MiddlewareFunction = (...args: T.RageFW_ServerArgs) =>
    Promise<RageFW_MiddlewareResponse>

type RageFW_MiddlewareResponse =
    | {
          success: boolean
          message?: string
      }
    | boolean
```

### Usage

`RageFW_MiddlewareFunction` can be imported to type your middlewares, eg.:

```ts
import { type RageFW_MiddlewareFunction } from '@entityseven/rage-fw-server'

const isPlayerAdmin: RageFW_MiddlewareFunction<'yourEventName'> = async (
  player,
  ...args
) => {
  if (player.adminLvl >= 2) {
    return true
  } else {
    return {
      success: false,
      message: 'You must have administrator rights for this action'
    }
  }
}

// or function-like if you want to omit some argument types or hoisting is required

async function isPlayerAdmin(
  player: PlayerMp,
  ...args
): ReturnType<RageFW_MiddlewareFunction<'yourEventName'>> {
  if (player.adminLvl >= 2) {
    return true
  } else {
    return {
      success: false,
      message: 'You must have administrator rights for this action'
    }
  }
}

fw.event.register(
  'yourEventName',
  async (player, ...args) => {
    // do an action which requires administrator rights
  },
  {
    middlewares: {
      executables: [isPlayerAdmin],
      onError: (e) => {} // notify player about missing permissions
    }
  }
)
```

### Implementation TL;DR (why no next()?)

Unfortunately in Client-side of Rage:MP every thrown `Error` or `Promise.reject` is not `catch`able, meaning a naughty error window will pop-up on player's screen. Due to this implementation variant we were forced to not throw `Errors` from middlewares and just return middleware result. Server-side can handle errors, but to keep everything consistent we stick up to this variant
